library(rjson)

net.alias.table <- function(alias.fname){
    temp <- read.table(alias.fname, sep = ",", header = T)
    rownames(temp) <- temp$name
    temp
}

net.genes.table <- function(net.fname, net.alias){
    ntemp <- read.table(net.fname, as.is = T, header = F)
    stmp <- (rbind(ntemp %>% select(V1, V2 = V3),
                   ntemp %>% select(V2 = V1, V1 = V3)) %>%
              count(V1, sort = T))
    stmp$Alias <- net.alias[stmp$V1, "Alias"]
    stmp$Id <- net.alias[stmp$V1, "Id"]
    colnames(stmp) <- c("Probe", "Degree", "Alias", "Id")
    stmp
}

emit.net.genes.table <- function(net.fname, alias.fname,
                                 out.dir, out.prefix, out.fname){
    nat <- net.alias.table(alias.fname)
    ngt <- net.genes.table(net.fname, nat)
    out.full.name <- paste(out.dir, out.prefix, out.fname, ".csv", sep = "")
    write.table(ngt, out.full.name, sep = ",", row.names = F)
}

merge.centratlity <- function(net.csv.file, cent.json.file,
                              out.csv.file){
  data.cent = rjson::fromJSON(file = cent.json.file)
  data.cent = data.frame(Probe = names(data.cent), Centrality = as.numeric(data.cent))
  rownames(data.cent) = data.cent$Probe
  net.data = read.table(net.csv.file, header = T,
                        stringsAsFactors = F, sep = ",")
  cat(nrow(data.cent), nrow(net.data))
  net.data$Centrality = data.cent[net.data$Probe, "Centrality"]
  write.table(net.data, out.csv.file, sep = ",", row.names = F)
}

adj.matrix.to.edge.list <- function(adj_mat, node_names){
    nnodes <- length(node_names)
    ldply(1:nnodes, function(i){
      ldply(i:nnodes, function(j){
        c(node1 = node_names[i], node2 = node_names[j],
          pcor = adj_mat[i,j])
      })
  })
}

merge.centratlity("../data/ggm-graphs/ggm-wholeplant.csv", "../data/ggm-graphs/combo5.json", "../data/ggm-graphs/ggm-whole-plant-centrality.csv")
merge.centratlity("../data/ggm-graphs/ggm-leaf.csv", "../data/ggm-graphs/combo10.json", "../data/ggm-graphs/ggm-leaf-centrality.csv")
merge.centratlity("../data/ggm-graphs/ggm-seedling.csv", "../data/ggm-graphs/combo15.json", "../data/ggm-graphs/ggm-seedling-centrality.csv")
merge.centratlity("../data/ggm-graphs/ggm-flower.csv", "../data/ggm-graphs/combo16.json", "../data/ggm-graphs/ggm-flower-centrality.csv")
merge.centratlity("../data/ggm-graphs/ggm-root.csv", "../data/ggm-graphs/combo21.json", "../data/ggm-graphs/ggm-root-centrality.csv")
merge.centratlity("../data/ggm-graphs/ggm-metabolism.csv", "../data/ggm-graphs/combo22.json", "../data/ggm-graphs/ggm-metabolism-centrality.csv")
merge.centratlity("../data/ggm-graphs/ggm-chemical.csv", "../data/ggm-graphs/combo91.json", "../data/ggm-graphs/ggm-chemical-centrality.csv")
merge.centratlity("../data/ggm-graphs/ggm-development.csv", "../data/ggm-graphs/combo92.json", "../data/ggm-graphs/ggm-development-centrality.csv")
merge.centratlity("../data/ggm-graphs/ggm-hormone.csv", "../data/ggm-graphs/combo94.json", "../data/ggm-graphs/ggm-hormone-centrality.csv")
merge.centratlity("../data/ggm-graphs/ggm-light.csv", "../data/ggm-graphs/combo95.json", "../data/ggm-graphs/ggm-light-centrality.csv")
merge.centratlity("../data/ggm-graphs/ggm-pathogen.csv", "../data/ggm-graphs/combo98.json", "../data/ggm-graphs/ggm-pathogen-centrality.csv")
merge.centratlity("../data/ggm-graphs/ggm-stress.csv", "../data/ggm-graphs/combo99.json", "../data/ggm-graphs/ggm-stress-centrality.csv")


merge.centratlity("../data/cor-graphs/cor-wholeplant.csv", "../data/cor-graphs/combo5.json", "../data/cor-graphs/cor-whole-plant-centrality.csv")
merge.centratlity("../data/cor-graphs/cor-leaf.csv", "../data/cor-graphs/combo10.json", "../data/cor-graphs/cor-leaf-centrality.csv")
merge.centratlity("../data/cor-graphs/cor-seedling.csv", "../data/cor-graphs/combo15.json", "../data/cor-graphs/cor-seedling-centrality.csv")
merge.centratlity("../data/cor-graphs/cor-flower.csv", "../data/cor-graphs/combo16.json", "../data/cor-graphs/cor-flower-centrality.csv")
merge.centratlity("../data/cor-graphs/cor-root.csv", "../data/cor-graphs/combo21.json", "../data/cor-graphs/cor-root-centrality.csv")
merge.centratlity("../data/cor-graphs/cor-metabolism.csv", "../data/cor-graphs/combo22.json", "../data/cor-graphs/cor-metabolism-centrality.csv")
merge.centratlity("../data/cor-graphs/cor-chemical.csv", "../data/cor-graphs/combo91.json", "../data/cor-graphs/cor-chemical-centrality.csv")
merge.centratlity("../data/cor-graphs/cor-development.csv", "../data/cor-graphs/combo92.json", "../data/cor-graphs/cor-development-centrality.csv")
merge.centratlity("../data/cor-graphs/cor-hormone.csv", "../data/cor-graphs/combo94.json", "../data/cor-graphs/cor-hormone-centrality.csv")
merge.centratlity("../data/cor-graphs/cor-light.csv", "../data/cor-graphs/combo95.json", "../data/cor-graphs/cor-light-centrality.csv")
merge.centratlity("../data/cor-graphs/cor-pathogen.csv", "../data/cor-graphs/combo98.json", "../data/cor-graphs/cor-pathogen-centrality.csv")
merge.centratlity("../data/cor-graphs/cor-stress.csv", "../data/cor-graphs/combo99.json", "../data/cor-graphs/cor-stress-centrality.csv")



merge.centratlity("../data/mi-graphs/mi-wholeplant.csv", "../data/mi-graphs/combo5.json", "../data/mi-graphs/mi-whole-plant-centrality.csv")
merge.centratlity("../data/mi-graphs/mi-leaf.csv", "../data/mi-graphs/combo10.json", "../data/mi-graphs/mi-leaf-centrality.csv")
merge.centratlity("../data/mi-graphs/mi-seedling.csv", "../data/mi-graphs/combo15.json", "../data/mi-graphs/mi-seedling-centrality.csv")
merge.centratlity("../data/mi-graphs/mi-flower.csv", "../data/mi-graphs/combo16.json", "../data/mi-graphs/mi-flower-centrality.csv")
merge.centratlity("../data/mi-graphs/mi-root.csv", "../data/mi-graphs/combo21.json", "../data/mi-graphs/mi-root-centrality.csv")
merge.centratlity("../data/mi-graphs/mi-metabolism.csv", "../data/mi-graphs/combo22.json", "../data/mi-graphs/mi-metabolism-centrality.csv")
merge.centratlity("../data/mi-graphs/mi-chemical.csv", "../data/mi-graphs/combo91.json", "../data/mi-graphs/mi-chemical-centrality.csv")
merge.centratlity("../data/mi-graphs/mi-development.csv", "../data/mi-graphs/combo92.json", "../data/mi-graphs/mi-development-centrality.csv")
merge.centratlity("../data/mi-graphs/mi-hormone.csv", "../data/mi-graphs/combo94.json", "../data/mi-graphs/mi-hormone-centrality.csv")
merge.centratlity("../data/mi-graphs/mi-light.csv", "../data/mi-graphs/combo95.json", "../data/mi-graphs/mi-light-centrality.csv")
merge.centratlity("../data/mi-graphs/mi-pathogen.csv", "../data/mi-graphs/combo98.json", "../data/mi-graphs/mi-pathogen-centrality.csv")
merge.centratlity("../data/mi-graphs/mi-stress.csv", "../data/mi-graphs/combo99.json", "../data/mi-graphs/mi-stress-centrality.csv")
