library("plyr")
library("dplyr")
library("ggplot2")
#source("io_env.R")

read.pipe.df <- function(in.file, ais = T){
    read.table(in.file, sep = "|", as.is = ais, header = T)
}

write.pipe.df <- function(in.df, in.file){
  write.table(in.df, file = in.file, sep = "|",
              row.names = F, quote = F, col.names = T)
}

read.comma.df <- function(in.file, ais = T){
    read.table(in.file, sep = ",", as.is = ais, header = T)
}

write.comma.df <- function(in.df, in.file){
  write.table(in.df, file = in.file, sep = ",",
              row.names = F, quote = F, col.names = T)
}

load.access.ids <- function(){
    read.table(accession.data.file, stringsAsFactors = F)
}

get.cel.filenames <- function(tdata, root_dir) {
    ddply(tdata, .(Database, Access), function(x){
        db_name <- x[1, "Database"]
        access_name <- x[1, "Access"]
        db_dir <- paste(root_dir, db_name, access_name,
                        sep = "/")
        data.frame(CEL=dir(db_dir, pattern = ".CEL$"),
                   stringsAsFactors = F)
    })
}

get.summary.data <- function(tdata, root_dir,
                              dest_prefix = "") {
    ddply(tdata, .(Database, Access), function(x){
        db_name <- x[1, "Database"]
        access_name <- x[1, "Access"]
        sum_file <- paste(root_dir, db_name, access_name,
                          paste(paste(dest_prefix, access_name, sep = ""),
                                "sum", sep = "."),
                          sep = "/")
        if(file.exists(sum_file) &
           file.info(sum_file)[1, "size"] > 0){
            read.table(sum_file, as.is = T)
        }
    })
}
