library(plyr)
library(ggplot2)
library(stringr)

correlation_matrix <- function(y){
  ngenes <- ncol(y)
  cmatrix <- matrix(0,ngenes,ngenes)
  for(i in 1:ngenes){
    for(j in i:ngenes){
      cmatrix[i,j] <- cor(y[,i],y[,j])
      cmatrix[j,i] <- cmatrix[i,j]
    }
  }
  cmatrix
}


system_run <- function() {

  stime <- system.time({
    lt1 <- ldply(1:nlength, .fun = function(i){
      n1 <- node_df[i,"node1"]
      n2 <- node_df[i,"node2"]
      if((i %% 1E6) == 0){
        temp_df <- data.frame(t(c(done = i)))
        write.table(temp_df, "status.txt", append = T, row.names = F,
                    col.names = F, quote = F)
      }
      c(node1 = n1, node2 = n2,
        cor = cor( tinge_data[,n1], tinge_data[,n2]))
    })
  }); print(stime); write.table(lt1, "cor_tinge-qn-ir6.0.cor.dat", quote = F, row.names = F, sep = "|")


  test.results <- read.table("gene_net_tinge-ath-qn-ir0.6.dat", sep = "|", as.is = T, header=T)
  cor.results <- test.results[,c("node1","node2")]
  cor.results$cor <- 0
  nlength <- nrow(node_df)
  stime <- system.time({
    l_ply(1:nlength, .fun = function(i){
      n1 <- cor.results[i,"node1"]
      n2 <- cor.results[i,"node2"]
      if((i %% 1E6) == 0){
        temp_df <- data.frame(t(c(done = i)))
        write.table(temp_df, "status.txt", append = T, row.names = F,
                    col.names = F, quote = F)
      }
      cor.results[i, 3] <- cor( x[,n1], x[,n2])
    })
  });
  print(stime);

  write.table(cor.results,
              "cor_tinge-qn-ir6.0.cor.dat", quote = F, row.names = F, sep = "|")

  k = 1
  for(i in 1:nlength){
    for(j in (i+1):nlength){
      k = k + 1
    }
  }

  for(i in 1:nlength) {
    n1 <- cor.results[i,"node1"]
    n2 <- cor.results[i,"node2"]
    if((i %% 1E6) == 0){
      temp_df <- data.frame(t(c(done = i)))
      write.table(temp_df, "status.txt", append = T, row.names = F,
                  col.names = F, quote = F)
    }
    cor.results[i, 3] <- cor( x[,n1], x[,n2])
  }
}

run.coexp.network <- function(exp.fname, out.fname){
    exp.tab <- read.table(exp.fname , skip = 3, as.is = T)
    rownames(exp.tab) <- exp.tab[,1]
    dt <- dim(exp.tab)
    exp.tab <- exp.tab[,3:dt[2]]
    exp.tab <- t(exp.tab)
    ctab <- cor(exp.tab)
    write.table(ctab, out.fname, sep = "\t")
}

run.coexp.dir <- function(dir.name){
    exp.fnames <- dir(dir.name, "*.exp", full.names = T)
    out.names <-
        str_c("../data/tis_cat4/pcor-combo-",
              unlist(str_replace(
                  unlist(llply(str_split(exp.fnames, "combo"),
                               function(x){ x[2] })), "exp", "dat")
                     ));
    llply(1:len(exp.fnames), function(i){
      gc();
      res <- run.coexp.network(exp.fnames[i], out.names[i]);
      gc();
      res
    })
}

count.exp.lines <- function(dir.name, replace.str){
    exp.fnames <- dir(dir.name, "*.exp", full.names = T)
    ldply(exp.fnames, function(x) {
        xid <- str_replace(str_replace(str_replace(x, ".exp", ""), t4_dir, ""),
                           replace.str, "")
        xnum <- as.numeric(unlist(
            str_split(system2("wc", args = c("-l", x),
                              stdout = TRUE, stderr = FALSE), " "))[1])
        c(id = xid, count = xnum - 3)
  })
}

cor.hist.file <- function(x, dir.out,
                          replace.str.suf = ".dat.elst.hist2",
                          replace.str.pre = "/pcor-combo-"){
    xid <- str_replace(str_replace(str_replace(x, replace.str.suf, ""),
                                   dir.out, ""),
                       replace.str.pre, "")
    rdf <- read.table(x, sep = "\t", header = T)
    rdf$id <- xid
    nodes <-  rdf$neither
    rdf$nodes <- rdf$neither
    nedges <- (nodes * (nodes - 1))/2.0
    rdf$edges <- nedges
    nedges[ nedges == 0] = 1
    rdf$density <- (as.numeric(rdf$above) + as.numeric(rdf$below))/ nedges
    rdf
}

cor.hist.dir <- function(hist.dir, ext.name = "*.hist2",
                         replace.str.suf = ".dat.elst.hist2",
                         replace.str.pre = "/pcor-combo-"){
    hist.fnames <- dir(hist.dir, ext.name, full.names = T)
    ldply(hist.fnames, function(x) {
        cor.hist.file(x, hist.dir,
                      replace.str.suf, replace.str.pre)
    });
}

cor.hist.plot <- function(all.hist, in.id,
                      x.limits = c(0.4, 0.9),
                      th.limits = c(0.4, 0.9),
                      in.density = 0.25,
                      font.name = "Arial",
                      font.size = 12){
    ggplot(all_hist[ (all_hist$id ==  in.id) &
                     (all_hist$threshold > th.limits[1]) &
                     (all_hist$threshold <= th.limits[2]) &
                     (all_hist$density < 0.25),],
       aes(x = threshold, y = density, colour = id)) +
    geom_point() +
    scale_x_continuous(limits = x.limits) +
    xlab("Threshold for Pearson Correlation") +
    ylab("Network Density") +
    labs(fill = "",colour = "") +
    geom_line() +
    theme_bw() +
    theme(text=element_text(family=font.name, size=font.size),
          legend.position = "none")
}

cor.hist.plot.pdf <- function(pdf.name, all.hist, in.id,  ...){
    p <- cor.hist.plot(all.hist, in.id, ...)
    ggsave(pdf.name, plot = p)
    p
}
