library('plyr')
library('GEOmetadb')
library('stringr')

src_base_dir <- './'
data_base_dir <- "C:/data/arrays"
aexp_data_base_dir <- "C:/data/arrays/ArrayExpress/"
geo_data_base_dir <- "C:/data/arrays/GEO/"
nasc_data_base_dir <- "C:/data/arrays/NASC/"
atgeneexpress_data_base_dir <-  "C:/data/arrays/TAIR/"
geo_database_path <- 'GEOmetadb.sqlite'
geo_gds_acess_file <- paste(src_base_dir,'geo_gse_gds_accession.txt',
                            sep="")
arrayexpress_trim_file <- paste(src_base_dir,
                                'arrayexpress_query_output_trim.txt',sep="")
arrayexpress_desc_file <- paste(src_base_dir,
                                'arrayexpress_descriptions.txt',
                                sep = "")
nasc_trim_file <-  paste(src_base_dir, "nasc_filtered.txt", sep =  "")
atgeneexpress_file <- paste(src_base_dir, "atgeneexpress_file_names.txt", sep = "")

tarfile_count_cels <- function(dir_name, tarfname){
  sys_cmd <- paste( 'for i in  `find ', dir_name, ' -name ',
                    tarfname,
                    '` ; do tar -tvf $i; done', sep="")
  print(sys_cmd)
  sys_out <- system(sys_cmd,intern=T)
  c(cel_file = sum(grepl("[.][cC][eE][lL]",sys_out)),
    cel_warn = sum(grepl("warning",sys_out)))
}


zipfile_count_cels <- function(dir_name, zipfname){

  sys_cmd <- paste( 'for i in  `find ', dir_name, ' -name ',
                    zipfname,
                    '` ; do unzip -vl $i; done', sep="")
  print(sys_cmd)
  sys_out <- system(sys_cmd,intern=T)
  c(cel_file = sum(grepl("[.][cC][eE][lL]$",sys_out)),
    cel_warn = sum(grepl("warning",sys_out)))
}

sample_count <- function(conn, gse_id){
  rs <- dbGetQuery(conn,paste("select gsm ",
                              "from gse_gsm where gse = '",gse_id,
                              "'",sep=""))
  nrow(rs)
}

get_date <- function(conn, gse_id){
  rs <- dbGetQuery(conn,paste("select gse, submission_date ",
                              "from gse where gse = '",gse_id,
                              "'",sep=""))
  if(nrow(rs) > 0){
    rs[1,'submission_date']
  } else {
    NA
  }
}

get_sample_data <- function(conn, gse_id){
  rs <- dbGetQuery(conn, paste("select gsm, organism_ch1, gpl from gsm ",
                               "where gsm in ",
                               "( select gsm from gse_gsm where gse = '",
                                gse_id,"' )", sep = ""))
  rs
}


get_supp_file <- function(conn, gse_id){
  rs <- dbGetQuery(conn,paste("select gse,title,gse,supplementary_file,summary ",
                              "from gse where gse = '",gse_id,
                              "'",sep=""))
  if(nrow(rs) > 0){
    rs[1,'supplementary_file']
  } else {
    NA
  }
}

report_gse_sample <- function(conn, gse_lst){
  ldply(gse_lst, function(x){
    sd <- get_sample_data(conn,x)
    org_count <- (sd[,'organism_ch1'] == "Arabidopsis thaliana")
    c(gse = x,
      sub_date = get_date(conn, x),
      organism = all(sd[,'organism_ch1'] == "Arabidopsis thaliana"),
      total = nrow(sd),
      oc = sum(org_count))
  })
}
geo_tarf_count_cels <- function(geo_name, dir_name = geo_name,
                                geo_base_dir = geo_data_base_dir){
  geo_tar_fname <- paste(geo_name, "_RAW.tar", sep = "")
  rval <- tarfile_count_cels(paste(geo_base_dir, dir_name, sep=""),
                             geo_tar_fname)
  rval
}

report_gse <- function(conn,r, count_cel_files = F){
  gse_id <- r[1,'access']
  rs <- dbGetQuery(conn,paste("select gse,title,gse,supplementary_file,summary ",
                              "from gse where gse = '",gse_id,
                              "'",sep=""))
  scount <- sample_count(conn, gse_id)
  url_base <- 'http://www.ncbi.nlm.nih.gov/gse/query/acc.cgi?acc='
  rval <- geo_tarf_count_cels(gse_id)
  if(nrow(rs) == 0){
    c(type = "Series",
      gse_id = gse_id, samples = NA, tile = NA,
      raw_file = NA,
      desc = NA, url = NA, rval, rval)
  } else {
    c(type = "Series",
      gse_id = rs[1,'gse'], samples = scount,
      title = rs[1,'title'], desc = rs[1,'summary'],
      raw_file = !is.na(rs[1,'supplementary_file']),
      url = paste(url_base, rs[1,'gse'],sep=""), rval)
  }
}

report_gds <- function(conn,r, count_cel_files = F){
  gds_id <- r[1,'access']
  rs <- dbGetQuery(conn,paste("select gds,title,gpl,gse,sample_count,description ",
                              "from gds where gds = '",gds_id,
                              "'",sep=""))
  gse_id <- rs[1,'gse']
  scount <- sample_count(conn, gse_id)
  rval <- geo_tarf_count_cels(gse_id, gds_id)
  url_base <- 'http://www.ncbi.nlm.nih.gov/sites/GDSbrowser?acc='
  if(nrow(rs) == 0){
    c(type = "Dataset",
      gse_id = gse_id, samples = NA, tile = NA,
      desc = NA, url = NA, rval)
  } else {
    c(type  = "Dataset",
      gse_id = rs[1,'gse'], samples = scount,
      title = rs[1,'title'], desc = rs[1,'description'],
      raw_file = !is.na(get_supp_file(conn,gse_id)),
      url = paste(url_base, rs[1,'gds'],sep=""), rval)
  }
}


geo_report <- function(conn, geo_accdf, ... ){
  ddply(geo_accdf, 'type', function(tdf){
    if(tdf[1,1] == 'DataSet') {
      ddply(tdf, 'access', function(r) {
        report_gds(conn, r, ...)
      })
    } else if(tdf[1,1] == 'Series') {
      ddply(tdf, 'access', function(r) {
        report_gse(conn, r, ... )
      })
    } else {
      data.frame(id = c(tdf[1,1]), count = NA, ret = NA)
    }
  })
}

arrayexp_zipf_count_cels <- function(aexp_name,
                                     aexp_base_dir = aexp_data_base_dir){
  aex_zip_fname <- paste(aexp_name, ".raw.[1-9]*.zip", sep = "")
  rval <- zipfile_count_cels(aexp_base_dir, aex_zip_fname)
  rval
}


arrayexp_report <- function(aexp_df,desc_lines,
                       access_col_name = 'Accession',
                       samples_col_name = 'Assays',
                       raw_file_col_name = 'Raw.Data',
                       count_cel_files = F){
  nd <- nrow(aexp_df)
  desc_list <- llply(desc_lines, function(x) {
    v <- strsplit(x,"[|]");
    rv <- c(v[[1]][2]);
    names(rv) <- v[[1]][1];
    rv
  })
  desc_vector <- unlist(desc_list)
  ldply(1:nd, function(i){
    exp_id <- as.character(aexp_df[i,access_col_name])
    url_base <- 'http://www.ebi.ac.uk/arrayexpress/experiments/'
    desc_val <- desc_vector[exp_id]
    names(desc_val) <-  'desc'
    raw_file_field <- as.character(aexp_df[i,raw_file_col_name])
    raw_file_detect <- raw_file_field == "TRUE" |
                    (!str_detect(raw_file_field,'Data is not available'))
    rval <- if(count_cel_files){
      arrayexp_zipf_count_cels(exp_id)
    } else {
      c(cel_file = NA, cel_warn = NA)
    }
    c(type = "ArrayExpress", access = exp_id,
      gse_id = NA,
      samples = aexp_df[i,samples_col_name],
      title = 'See Desc',
      desc = desc_val ,
      raw_file = raw_file_detect,
      url = paste(url_base, exp_id, "/", sep=""),
      rval
      )
  })
}

geo_report_wflow <- function(geo_file  = geo_gds_acess_file,
                             col.names = c('type','access','id'),
                             colClasses = c('factor','character','character'),
                             header = F, ... ){
  geo_table <-
    if(header == F){
      read.table(geo_file, col.names = c('type','access','id'),
                 colClasses=c('factor','character','character'),
                 sep='|')
    } else {
      read.table(geo_file, header = T,
                 sep='|')
    }
  conn <- dbConnect(SQLite(), geo_database_path)
  report <- geo_report(conn,geo_table, ... )
  dbDisconnect(conn)
  report
}
gse_arrayexp_missing <-  function(aefile = arrayexpress_trim_file,
                                  field_name = 'Accession',
                                  geofile =  geo_gds_acess_file){
  aextable <- read.table(aefile,
                         sep='|',header=T, comment.char = "")
  geo_table <-  read.table(geofile,
                           col.names = c('type','access','id'),
                           colClasses=c('factor','character','character'),
                           sep='|')
  geo_access <- geo_table[,'access']
  aexp_access <- llply(aextable[,field_name], function(x){
    gse_id <- unlist(strsplit(as.character(x),'-'))[3]
    c(paste('GDS',gse_id,sep = ""),paste('GSE',gse_id,sep = ""))
  })
  aexp_access <- unlist(aexp_access)
  geo_present <- ddply(geo_table,c("type","access"), function(x){
    if( x[1,2] %in% aexp_access ) {
      c(x[1,1],x[1,2], val = T)
    } else {
      c(x[1,1],x[1,2], val = F)
    }
  })
  geo_present
}

arrayexp_gse_missing <-  function(aefile = arrayexpress_trim_file,
                                  field_name = 'Accession',
                                  geofile =  geo_gds_acess_file){
  aextable <- read.table(aefile,
                         sep='|',header=T, comment.char = "")
  geo_table <-  read.table(geofile,
                           col.names = c('type','access','id'),
                           colClasses=c('factor','character','character'),
                           sep='|')
  geo_access <- geo_table[,'access']
  geo_present <- ldply(aextable[,field_name], function(x){
    gse_id <- unlist(strsplit(as.character(x),'-'))[3]
    if( paste('GDS',gse_id,sep = "") %in% geo_access |
          paste('GSE',gse_id,sep = "") %in% geo_access) {
      c(id = as.character(x), val = T)
    } else {
      c(id = as.character(x), val = F)
    }
  })
  geo_present[geo_present$val == F,]
}

arrayexp_gse_missing <-  function(aefile = arrayexpress_trim_file,
                                  field_name = 'Accession',
                                  geofile =  geo_gds_acess_file){
  aextable <- read.table(aefile,
                        sep='|',header=T, comment.char = "")
  geo_table <-  read.table(geofile,
                           col.names = c('type','access','id'),
                           colClasses=c('factor','character','character'),
                           sep='|')
  geo_access <- geo_table[,'access']
  geo_present <- ldply(aextable[,field_name], function(x){
    gse_id <- unlist(strsplit(as.character(x),'-'))[3]
    if( paste('GDS',gse_id,sep = "") %in% geo_access |
        paste('GSE',gse_id,sep = "") %in% geo_access) {
      c(id = as.character(x), val = T)
    } else {
      c(id = as.character(x), val = F)
    }
  })
  geo_present[geo_present$val == F,]
}

arrayexp_report_wflow <- function(aex_file = arrayexpress_trim_file,
                                  ...){
  aextable <- read.table(aex_file,
                        sep='|',header=T,comment.char = "")
  fconn <- file(arrayexpress_desc_file)
  desc_lines <- readLines(fconn)
  close(fconn)
  areport <- arrayexp_report(aextable,desc_lines,...)
  # 583 rows after previous ones in the arrayexpress trim file
  #areport <- areport[1:583,]
  areport <- areport[!(grepl("*NASC*",areport$access)), ]
  # We are doing the other way round.. so, this is not required
  #gcommon <- arrayexp_gse_missing()
  #areport[ areport$access %in% gcommon$id, ]
  areport
}

nasc_list_cel_files <- function(exp_name,
                                nasc_base_dir = nasc_data_base_dir
                                ){
  sys_cmd <- paste( 'ls -1 ', paste(nasc_base_dir, exp_name, sep = "") )
  print(sys_cmd)
  sys_out <- system(sys_cmd,intern=T)
  c(cel_file = sum(grepl("[.][cC][eE][lL]",sys_out)),
    cel_warn = sum(grepl("warning",sys_out)))

}

nasc_report_wflow <- function(nasc_file_name = nasc_trim_file,
                              header = F,
                              nasc_id_col_name = "nasc_exp_id",
                              nasc_desc_col_name = "nasc_desc",
                              colNames = c("nasc_exp_id", "nasc_desc"),
                              colClasses = c("character", "character")){
  # output coluns are
  #access|type|gse_id|samples|title|desc|raw_file|url|cel_file|cel_warn
  nasc_table <-  if(header == T){
    read.table(nasc_file_name, sep = "|", header = T, as.is = T)
  } else {
    read.table(nasc_file_name, col.names = colNames,
               colClasses= colClasses, sep='|', as.is = T)
  }
  colIdx <- match( nasc_id_col_name, colnames(nasc_table))
  descIdx <- match( nasc_desc_col_name, colnames(nasc_table))
  nasc_report <- ddply(nasc_table , nasc_id_col_name, function(r){
    exp_id <- str_replace( r[1,colIdx], "Exp", "")
    access_id <- paste( "NASC", exp_id, sep = "-")
    title_desc <- r[1,descIdx]
    rval <- nasc_list_cel_files( r[1, colIdx] )
    nasc_url <- paste(
      "http://affymetrix.arabidopsis.info/narrays/experimentpage.pl",
      "?experimentid=", exp_id, sep= "")
    c(access = access_id, type = "NASC", gse_id = NA,
      samples = NA, title = "See Desc", desc = title_desc, raw_file = NA,
      url = nasc_url, rval)
  })
}


atgeneexpress_list_cel_files <- function( atgeneexpress_name,
                                          atgeneexpress_base_dir = atgeneexpress_data_base_dir ){
  zipfile_count_cels(paste(atgeneexpress_base_dir, atgeneexpress_name, sep = ""), "*.zip")

}

atgeneexpress_report_wflow <- function( atfile_name = atgeneexpress_file,
                                        tair_col_name = "experiment"){

  # output coluns are
  #access|type|gse_id|samples|title|desc|raw_file|url|cel_file|cel_warn
  atgeneexpress_table <-  read.table(atfile_name, header = T, sep='|', as.is=T)
  ddply(atgeneexpress_table, tair_col_name, function(r){
    me_id <- r[1, 2]
    rval <- atgeneexpress_list_cel_files(me_id)
    atgeneexpress_url <- NA
    c(access = me_id, type = "AtGeneExpress", gse_id = NA,
      samples = NA, title = NA, desc = NA, raw_file = NA,
      url = atgeneexpress_url, rval)
  })

}
