base_dir <- '/data/arrays/ArrayExpress/'
#base_dir <- "/Users/srirampc/work/phd/research/ni-review/metadata/"

test_ids = c(4378, 3934)
library('plyr')
library('GEOmetadb')
library('RCurl')

test_df <- data.frame(type = c('DataSet', 'Series', 'Series'),
                      access = c('GDS3934', 'GSE47092', 'GSE44405'))

run_download2 <- function(src_url, dest_file) {
  paste('wget',supp_file,'-O',dest_file, sep=" ")
  system(cmd)
}

run_download <- function(dir_name,supp_file,
                         log_file = NULL,
                         dest_bname = NULL,
                         dest_base_dir = base_dir){
  new_dir <- paste(dest_base_dir,'/',dir_name,'/',sep = "")
  dest_file <- if(is.null(dest_bname)) {
    bname <- basename(supp_file)
    paste(new_dir,bname,sep="")
  } else {
    paste(new_dir,dest_bname,sep="")
  }
  cmd <- if(is.null(log_file)){
    paste('wget',supp_file,'-O',dest_file,sep=" ")
  } else {
    paste('wget',supp_file,'-O',dest_file, "-a", log_file, sep=" ")
  }
  print(cmd)
  dir.create(new_dir)
  system(cmd)
}

get_geo_soft_url <- function(GEO){
    geotype <- toupper(substr(GEO, 1, 3))
    GEO <- toupper(GEO)
    stub = gsub("\\d{1,3}$", "nnn", GEO, perl = TRUE)
    if (geotype == "GDS") {
        gdsurl <- "ftp://ftp.ncbi.nlm.nih.gov/geo/datasets/%s/%s/soft/%s"
        myurl <- sprintf(gdsurl, stub, GEO, paste0(GEO, ".soft.gz"))
        myurl
    } else if (geotype == "GSE") {
        gseurl <- "ftp://ftp.ncbi.nlm.nih.gov/geo/series/%s/%s/soft/%s"
        myurl <- sprintf(gseurl, stub, GEO, paste0(GEO, "_family.soft.gz"))
        myurl
    } else {
      NA
    }
}
download_gse <- function(conn,r,dir_name = F,
                         run = "wet"){
  gse_id <- r[1,'access']
  rs <- dbGetQuery(conn,paste("select gse,supplementary_file ",
                              "from gse where gse = '",gse_id,
                              "'",sep=""))
  if(nrow(rs) == 0){
    c(id = gse_id, count =0, ret = NA)
  }else if(nrow(rs) == 1 & is.na(rs[1,'supplementary_file'])) {
    c(id = gse_id, count = -1, ret = NA)
  }else{
    for(i in 1:nrow(rs)){
      supp_file <- rs[i,'supplementary_file']
      print(supp_file)
      dir_name <- if(dir_name == F) {
        gse_id
      } else {
        dir_name
      }
      if(run == "wet") {
        supp_vec <- unlist(strsplit(supp_file,';\t'))
        rv <- 0
        for(i in 1:length(supp_vec)){
          print(supp_vec[i])
          lrval <- run_download(dir_name, supp_vec[i])
          if(lrval != 0){
            rv <- lrval
          }
        }
      } else {
        rv <- 0
      }
    }
    c(id = gse_id,count = nrow(rs), ret = rv)
  }
}

download_gds <- function(conn,r,run = "wet"){
  gds_id <- r[1,'access']
  rs <- dbGetQuery(conn,paste("select gds,title,gpl,gse ",
                              "from gds where gds = '",gds_id,
                              "'",sep=""))
  if(nrow(rs) == 0){
    c(id = gds_id, count = 0, ret = NA)
  } else {
    gse_id <- rs[1,'gse']
    geo_d <- download_gse(conn, data.frame( type = c('Series'),
                                           access = c(gse_id)),
                          dir_name = gds_id,
                          run = run)
    c(id = gds_id, geo_d[2], geo_d[3])
  }
}


download_geo <- function(conn, geo_accdf, run="wet"){
  ddply(geo_accdf, 'type', function(tdf){
    if(tdf[1,1] == 'DataSet') {
      ddply(tdf, 'access', function(r) {
        download_gds(conn, r,run = run)
      })
    } else if(tdf[1,1] == 'Series') {
      ddply(tdf, 'access', function(r) {
        download_gse(conn, r,run = run)
      })
    } else {
      data.frame(id = c(tdf[1,1]), count = NA, ret = NA)
    }
  })
}

geo_wflow <- function(){
  geo_table <- read.table('geo_gse_gds_accession.txt',
                          col.names = c('type','access','id'), colClasses=c('factor','character','character'),sep='\t')
}
library('stringr')

arrayexp_metadata_url <- function(aexp_name){
  # example "http://www.ebi.ac.uk/arrayexpress/files/E-MEXP-3726/E-MEXP-3726.sdrf.txt"
  url_prefix <- "http://www.ebi.ac.uk/arrayexpress/files"
  paste(url_prefix, aexp_name, paste(aexp_name, "sdrf.txt", sep = "."),
        sep = "/")
}

arrayexp_metadata_download <- function(aexp_df,
                              dest_dir = "array_express",
                              exp_col_name = "Accession"){
  nd <- nrow(aexp_df)
  ldply(1:nd, function(i){
    exp_id <- as.character(aexp_df[i,exp_col_name])
    print(exp_id)
    meta_url <- arrayexp_metadata_url(exp_id)
    print(meta_url)
    brname <- basename(meta_url)
    ret <- c(exp_id,0,brname)
    ret[2] <- run_download(exp_id, meta_url, dest_base_dir = dest_dir)
    ret
  })
}

nasc_meta_url <- function(nasc_id){
  url_prefix <- "http://affymetrix.arabidopsis.info/narrays/experimentpage.pl?experimentid="
  nid <- unlist(strsplit(nasc_id, "Exp"))[2]
  paste(url_prefix,nid, sep = "")
}

nasc_metadata_download <- function(nasc_df,
                                   dest_dir = "nasc",
                                   exp_col_name = "Accession"){
  nd <- nrow(nasc_df)
  meta_log_file <- paste(base_dir,dest_dir, "meta.log", sep = "/")
  dest_base_dir <- paste(base_dir,dest_dir, sep = "/")
  ldply(1:nd, function(i){
    nasc_id <- as.character(nasc_df[i,exp_col_name])
    print(nasc_id)
    meta_url <- nasc_meta_url(nasc_id)
    print(meta_url)
    dest_file <- paste(nasc_id, "html", sep = ".")
    ret <- c(nasc_id,0,dest_file)
    ret[2] <- run_download(nasc_id, meta_url,
                           log_file = meta_log_file,
                           dest_bname = dest_file,
                           dest_base_dir = dest_base_dir)
    ret
  })
}
arrayexp_download <- function(aexp_df,
                              exp_col_name = "Accession",
                              raw_file_col_name = "Raw.Data",
                              processed_file_col_name = "Processed.Data"){
  nd <- nrow(aexp_df)
  ldply(1:nd, function(i){
    exp_id <- as.character(aexp_df[i,exp_col_name])
    raw_file <- as.character(aexp_df[i,raw_file_col_name])
    print(raw_file)
    processed_file <- as.character(aexp_df[i,processed_file_col_name])
    brname <- basename(raw_file)
    bpname <- basename(processed_file)
    ret <- c(exp_id,F,F,0,0,brname,bpname)
    if(str_detect(brname,'.zip')){
      ret[4] <- run_download(exp_id,raw_file)
      ret[2] <- T
    }
    if(str_detect(bpname,'.zip')){
      ret[5] <- run_download(exp_id,processed_file)
      ret[3] <- T
    }
    ret
  })
}

arrayexp_wflow <- function(){
  aextable = read.table('arrayexpress_query_output_trim.txt',
                        sep='|',header=T)
}

tair_dir_listing <- function(tair_df){
  ddply(tair_df,'tair_name', function(trow) {
    tair_name <- trow[1]
    durl <- trow[2]
    print(durl)
    lstr <- RCurl::getURL(durl,dirlistonly=T)
    lspl <- strsplit(lstr,'\n')
    xdf <- ldply(lspl[[1]], function(x) {
      xc <-  str_count(x, "<tr class=\\\"entry\\\"><td class=\\\"icon\\\"><a href=" );
      c(string = x, count = xc)
    } )
    sstr <- xdf[xdf$count == 1,]$string
    my_df <- ldply(sstr,function(x) {
      c(count = length(sstr), file_name = str_extract(x,"([A-Za-z0-9_]+.zip)"))
    })
  })
}

tair_download <- function(tair_df, tair_fnames_df){
  row.names(tair_df) <- tair_df$tair_name
  ddply(tair_fnames_df,'file_name', function(trow) {
    tname <- as.character(trow[1])
    root_url <- tair_df[tname,'durl']
    fname <- as.character(trow[3])
    final_url <- paste(root_url,fname,sep="")
    c(tair_name = tname, status = run_download(tname,final_url))
  })
}

tair_wflow <- function(){
  tair_url_df <- read.table('atgeneexpress_dir_url.txt',
                            sep="|",col.names=c('tair_name','durl'),
                            colClasses=c('character','character'))
  tair_wflow_step1 <- function() {
    tair_fnames <- tair_dir_listing(tair_url_df);
    write.table(tair_fnames,"atgeneexpress_file_names.txt")
  }
  tair_wflow_step2 <- function() {
    tair_fnames_df <- read.table('atgeneexpress_file_names.txt',as.is=T)
    tair_fnames_df <- tair_fnames_df[!is.na( tair_fnames_df$file_name ),]
    tair_download_run <- tair_download(tair_df,tair_fnames_df)
    tair_download_run
  }
  tair_wflow_step1()
  tair_wflow_step2()
}

nasc_dir_files <- function(nasc_name) {
  root_url <- 'ftp://arabidopsis.info/pub/NASCArrays/Data/By_Experiment_ID'
  durl <- paste(root_url, '/', nasc_name, sep = "")
  print(durl)
  lstr <- RCurl::getURL(durl,dirlistonly=T)
  lspl <- strsplit(lstr,'\n')
  xdf <- ldply(lspl[[1]], function(x) {
    #print(x)
    xc <-  str_count(x, paste("<A HREF=\\\"",nasc_name, "/",sep="") );
    c(string = x, count = xc)
  } )
  sstr <- xdf[xdf$count > 0,]$string
  ldply(sstr,function(x) {
      #print(x)
      if(!is.na(str_extract(x, "([^/ \t\"]+.CEL.gz)"))){
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.CEL.gz)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.CHP.gz)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.CHP.gz)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.RPT.gz)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.RPT.gz)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.RPT)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.RPT)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.CEL)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.CEL)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.CHP)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.CHP)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.TXT)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.TXT)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.JPG)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.JPG)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.DAT)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.DAT)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.EXP)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.EXP)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.DTT)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.DTT)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.AUDIT)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.AUDIT)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.cel)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.cel)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.txt.gz)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.txt.gz)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.tar.gz)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.tar.gz)"))
      } else if (!is.na(str_extract(x,"([^/ \t\".]+.zip)"))) {
        c(count = length(sstr), file_name = str_extract(x,"([^/ \t\"]+.zip)"))
      } else {
        c(count = length(sstr), file_name = NA)
      }
    })
}
nasc_dir_listing <- function(nasc_df){
  ddply(nasc_df,'nasc_name', function(trow) {
    nasc_name <- trow[1,'nasc_name']
    nasc_dir_files(nasc_name)
  })
}

nasc_download <- function(nasc_fnames_df){
  root_url <- 'ftp://arabidopsis.info/pub/NASCArrays/Data/By_Experiment_ID/'
  ddply(nasc_fnames_df, c('nasc_name','file_name'), function(trow) {
    tname <- as.character(trow[1,1])
    fname <- as.character(trow[1,3])
    #print(tname)
    #print(fname)
    fname <- gsub("\\(","\\\\(",fname)
    fname <- gsub("\\)","\\\\)",fname)
    final_url <- paste(root_url,tname,"/",fname,sep="")
    c(nasc_name = tname, status = run_download(tname,final_url))
  })
}

nasc_wflow <- function(nasc_file_name = 'nasc_all.txt', colNames = c('nasc_name')){
  #nasc_data1 <- read.table('nasc_all.txt',as.is=T,header=F,col.names=c('nasc_name'))
  nasc_data1 <- read.table(nasc_file_name,as.is=T,header=F,col.names=colNames)
  nasc_data1$count <- 0
  nasc_dir1 <- nasc_dir_listing(nasc_data1)
  list(nasc_dir1, nasc_download(nasc_dir1))
}

array_express_check <- function() {
  check_base_dir <- "./test/"
  aex_check <- read.table("arrayexpress_sdrf.org",sep=" | ")
  for(i in 1:nrow(aex_check)){
    run_download(check_base_dir, as.character(aex_check[i,3]))
  }
  t <- ldply(1:nrow(aex_check), function(r) {
    fname <- basename(str_trim(as.character(aex_check[r,3])))
    fconn <- file(paste(check_base_dir,fname,sep=""))
    desc_lines <- readLines(fconn)
    close(fconn)
    gr_count <- sum(grepl(x = desc_lines, pattern = "Arabidopsis thaliana"))
    sr_count <- as.numeric(aex_check[r,2])
    c(fname, sr_count, gr_count, gr_count / sr_count)
  })
}
