library(GeneNet)
make.mat.file <- function(exp.fname){
    exp_tab <- read.table(exp.fname , skip = 3, as.is = T)
    rownames(exp_tab) <- exp_tab[,1]
    dt <- dim(exp_tab)
    exp_tab <- exp_tab[,3:dt[2]]
    exp_tab <- t(exp_tab)
    write.table(exp_tab, paste(exp.fname, ".mat", sep = ""))
}

make.mat.dir <- function(exp.dir){
    exp.fnames <- dir(exp.dir, "*.exp$", full.names = T)
    ldply(exp.fnames, make.mat.file)
}

run.ggm.network <- function(exp.fname, out.fname){
   exp_tab <- read.table(exp.fname , skip = 3, as.is = T)
   rownames(exp_tab) <- exp_tab[,1]
   dt <- dim(exp_tab)
   exp_tab <- exp_tab[,3:dt[2]]
   exp_tab <- t(exp_tab)
   inferred.pcor <- ggm.estimate.pcor(exp_tab)
   write.table(inferred.pcor, out.fname, sep = "\t")
}

run.estimate.pcor <- function(mat.fname, save.fname, out.fname){
    exp_tab = read.table(mat.fname)
    inf.pcor = ggm.estimate.pcor(exp_tab)
    saveRDS(inf.pcor, file = save.fname)
    test.results <- ggm.test.edges(inf.pcor)
    write.table(test.results, out.fname)
}

test.ggm.run <- function(){
    data(ecoli)
    exp_tab = ecoli
    inf.pcor = ggm.estimate.pcor(exp_tab)
    test.results <- ggm.test.edges(inf.pcor)
    write.table(test.results, "ecoli0.txt")
}

run.combos <- function(dir.name, combos.torun){
    for(x in combos.torun){
        in.fname <- paste(dir.name, "/tinge-ATH1-20140221-qn-iqr0.60-combo",
                          x, ".exp.mat", sep = "")
        sav.fname <- paste(dir.name, "/combo",
                           x, ".inf.rds", sep = "")
        out.fname <- paste("ggm-combo", x, "-results.txt")
        run.estimate.pcor(in.fname, sav.fname, out.fname)
    }
}
