source("preprocess.R")

library("plyr")
library("tools")
library("RColorBrewer")
library("affyPLM")

gen_expmd5_df <- function() {
  final_count <- count_final_dir(c("NASC","ArrayExpress","TAIR","GEO"))
  odf <- ldply(1:nrow(final_count), function(i) {
    cel_fname <- paste(final_dir, do.call(paste,c(final_count[i,], sep = "/")),
                       sep = "/")
    rds_fname <- paste(cel_fname, "rds", sep = ".")
    md5v <- as.character(md5sum(rds_fname))
    cstat <- c(db = final_count[i,1], id =  final_count[i,2],
               fname = final_count[i,3],
               md5 = md5v )
  })
  todf <- ddply( odf, .(md5), function(x){ if(nrow(x) > 1) {x} })
}

gen_exptable_df <- function(final_count) {
  ldply(1:nrow(final_count), function(i) {
    cel_fname <- paste(final_dir, do.call(paste,c(final_count[i,], sep = "/")),
                       sep = "/");
    out_fname <- paste(cel_fname, "rds", sep = ".")
    celf <- ReadAffy(filenames = cel_fname);
    rv <- saveRDS(exprs(celf), out_fname)
    md5v <- as.character(md5sum(out_fname))
    cstat <- c(db = final_count[i,1], id =  final_count[i,2],
               fname = final_count[i,3],
               md5 = md5v, dim = dim(exprs(celf)) )
    write.table(data.frame(t(cstat)) ,"exptable_run_progress.txt",
                col.names = F, row.names = F, append = T,sep = "|",
                quote = F)
    cstat
  })
}

generate_exptable <- function() {
  final_count <- count_final_dir(c("NASC","ArrayExpress","TAIR","GEO"))
  fcr <- count_cel_files(final_count)
  odf <- gen_exptable_df(final_count)
  write.table(odf, "exptable_results.txt", row.names = F, append = T,
              sep = "|", quote = F)
}

gen_cel_image <- function(cel_fname, out_fname) {
    rdb_palette <- colorRampPalette(brewer.pal(9, "RdBu"))(32)
    celf <- ReadAffy(filenames = cel_fname)
    pdf(out_fname)
    image(celf[,1], col=rdb_palette)
    dev.off()
}

gen_cel_image_df <- function(source_dir, fdr) {
  ldply(1:nrow(fdr), function(i) {
    cel_fname <- paste(source_dir, do.call(paste,c(fdr[i,], sep = "/")),
                       sep = "/");
    out_fname <- paste(cel_fname, "pdf", sep = ".")
    out_fname <- gsub("%", "p", out_fname)
    out_fname <- gsub(" ", "s", out_fname)
    print(out_fname)
#    gen_cel_image(cel_fname, out_fname)
#    rdb_palette <- colorRampPalette(brewer.pal(9, "RdBu"))(32)
#    celf <- ReadAffy(filenames = cel_fname)
#    pdf(out_fname)
#    image(celf[,1], col=rdb_palette)
#    dev.off()
    result <- c(fdr[i,], image = out_fname)
    #print(str(unlist(result)))
    write.table(data.frame(t(unlist(result))) ,"cel_run_progress.txt",
                col.names = F, row.names = F, append = T, sep = "|",
                quote = FALSE)
    data.frame(t(result))
  })
}

gen_cel_dir <- function(cel_dir){
   cel_files <- dir(cel_dir, "*.CEL$")
   nfiles <- length(cel_files)
   for(i in 1:nfiles){
     cel_fname <- paste(cel_dir, cel_files[i], sep = "/")
     pdf_fname <- paste(cel_fname, "pdf", sep = ".")
     print(cel_fname)
     print(pdf_fname)
     gen_cel_image(cel_fname, pdf_fname)
   }
}

generate_cel_images <- function(source_dir) {
  fdr <- count_final_dir(c("NASC","ArrayExpress","TAIR","GEO"))
  odf <- gen_cel_images(fdr)
  write.table(odf, "exptable_results.txt", row.names = F, append = T,
              sep = "|", quote = F)
}

update_todf <- function(todf, rmids){
  ddply( todf[ !(todf$id %in% rmids), ], .(md5), function(x){ if(nrow(x) > 1) {x} })
}

nuse_cel_files <- function(data_df, analysis_root_dir,
                           final_root_dir) {
  missing_cel_df <- ddply(data_df, .(Database, Access), function(r){
    db_name <- r[1 , "Database"]
    access_name <- r[1, "Access"]
    fail_file <- paste(analysis_root_dir, db_name, access_name,
                       paste(access_name, "sum", sep = "."), sep = "/")
    read.table(fail_file, as.is = T, col.names = c("cel_name"))
  })
  nfiles <- nrow(missing_cel_df)
  ldply(1:nfiles, function(i){
     cel_fname <- paste(analysis_root_dir, do.call(paste, c(missing_cel_df[i,], sep = "/")),
                        sep = "/")
     pdf_fname <- paste(final_root_dir, do.call(paste, c(missing_cel_df[i,], sep = "/")),
                        sep = "/")
     pdf_fname <- gsub("%", "p", pdf_fname)
     pdf_fname <- gsub(" ", "s", pdf_fname)
     pdf_fname <- paste(pdf_fname, "pdf", sep = ".")
     print(pdf_fname)
     pdf_dir <- dirname(pdf_fname)
     system(paste("mkdir -p", pdf_dir, sep = " "))
     gen_cel_image(cel_fname, pdf_fname)
     c(Database =  missing_cel_df[i, "Database"], Access = missing_cel_df[i, "Access"],
       cel_name = missing_cel_df[i, "cel_name"],
       celfname = cel_fname, pdfname = pdf_fname)
   })
}
